from typing import Any, TypedDict, final, overload, type_check_only
from typing_extensions import Unpack

import numpy as np
import numpy.typing as npt
import optype.numpy as onpt
from scipy._typing import EnterSelfMixin, FileLike, FileModeRW

__all__ = ["FortranEOFError", "FortranFile", "FortranFormattingError"]

@type_check_only
@final
class _DTypeKwargs(TypedDict):
    dtype: npt.DTypeLike

class FortranEOFError(TypeError, OSError): ...
class FortranFormattingError(TypeError, OSError): ...

class FortranFile(EnterSelfMixin):
    def __init__(self, /, filename: FileLike[bytes], mode: FileModeRW = "r", header_dtype: npt.DTypeLike = ...) -> None: ...
    def close(self, /) -> None: ...
    def write_record(self, /, *items: npt.ArrayLike) -> None: ...
    @overload
    def read_record(self, /, *dtypes: npt.DTypeLike) -> onpt.Array[tuple[int], np.void]: ...
    @overload
    def read_record(self, /, *dtypes: npt.DTypeLike, **kwargs: Unpack[_DTypeKwargs]) -> onpt.Array[tuple[int], np.void]: ...
    @overload
    def read_ints(self, /) -> onpt.Array[tuple[int], np.int32]: ...
    @overload
    def read_ints(self, /, dtype: onpt.AnyIntegerDType) -> onpt.Array[tuple[int], np.integer[Any]]: ...
    @overload
    def read_ints(self, /, dtype: npt.DTypeLike) -> onpt.Array[tuple[int]]: ...
    @overload
    def read_reals(self, /) -> onpt.Array[tuple[int], np.float64]: ...
    @overload
    def read_reals(self, /, dtype: onpt.AnyFloatingDType) -> onpt.Array[tuple[int], np.floating[Any]]: ...
    @overload
    def read_reals(self, /, dtype: npt.DTypeLike) -> onpt.Array[tuple[int]]: ...
