from typing import Literal, TypeAlias, overload
from typing_extensions import TypeVar

import numpy as np
import numpy.typing as npt
import optype.numpy as onpt
import scipy._typing as spt

__all__ = [
    "block_diag",
    "circulant",
    "companion",
    "convolution_matrix",
    "dft",
    "fiedler",
    "fiedler_companion",
    "hadamard",
    "hankel",
    "helmert",
    "hilbert",
    "invhilbert",
    "invpascal",
    "kron",
    "leslie",
    "pascal",
    "toeplitz",
]

_SCT = TypeVar("_SCT", bound=np.generic, default=np.generic)

_Matrix: TypeAlias = onpt.Array[tuple[int, int], _SCT]
_Kind: TypeAlias = Literal["symmetric", "upper", "lower"]

###

# TODO(jorenham): transparent dtypes
def toeplitz(c: npt.ArrayLike, r: npt.ArrayLike | None = None) -> _Matrix: ...
def circulant(c: npt.ArrayLike) -> _Matrix: ...
def hankel(c: npt.ArrayLike, r: npt.ArrayLike | None = None) -> _Matrix: ...
def hadamard(n: spt.AnyInt, dtype: npt.DTypeLike = ...) -> _Matrix: ...
def leslie(f: npt.ArrayLike, s: npt.ArrayLike) -> _Matrix: ...
def kron(a: npt.ArrayLike, b: npt.ArrayLike) -> _Matrix: ...
def block_diag(*arrs: npt.ArrayLike) -> _Matrix: ...
def companion(a: npt.ArrayLike) -> _Matrix: ...
def helmert(n: spt.AnyInt, full: bool = False) -> _Matrix[np.float64]: ...
def hilbert(n: spt.AnyInt) -> _Matrix[np.float64]: ...
def fiedler(a: npt.ArrayLike) -> _Matrix: ...
def fiedler_companion(a: npt.ArrayLike) -> _Matrix: ...
def convolution_matrix(a: npt.ArrayLike, n: spt.AnyInt, mode: spt.CorrelateMode = "full") -> _Matrix: ...

#
@overload
def invhilbert(n: spt.AnyInt, exact: Literal[False] = False) -> _Matrix[np.float64]: ...
@overload
def invhilbert(n: spt.AnyInt, exact: Literal[True]) -> _Matrix[np.int64] | _Matrix[np.object_]: ...
@overload
def pascal(n: spt.AnyInt, kind: _Kind = "symmetric", exact: Literal[True] = True) -> _Matrix[np.uint64 | np.object_]: ...
@overload
def pascal(n: spt.AnyInt, kind: _Kind = "symmetric", *, exact: Literal[False]) -> _Matrix[np.float64]: ...
@overload
def pascal(n: spt.AnyInt, kind: _Kind, exact: Literal[False]) -> _Matrix[np.float64]: ...
@overload
def invpascal(n: spt.AnyInt, kind: _Kind = "symmetric", exact: Literal[True] = True) -> _Matrix[np.int64 | np.object_]: ...
@overload
def invpascal(n: spt.AnyInt, kind: _Kind = "symmetric", *, exact: Literal[False]) -> _Matrix[np.float64]: ...
@overload
def invpascal(n: spt.AnyInt, kind: _Kind, exact: Literal[False]) -> _Matrix[np.float64]: ...
def dft(n: spt.AnyInt, scale: Literal["sqrtn", "n"] | None = None) -> _Matrix[np.complex128]: ...
